import time
import struct

servo_address = r".\Data\Data_tmp"


class Robot:

    def __init__(self, server):
        self.server = server

        self.client = None
        self.adder = None
        self.status = 0

        self.move_cmd = 3
        self.back_data = []
        self.time_start = 0

        self.position = [0, 500, 500, 500, 500, 500, 500]

    def connect(self):
        while 1:
            s_client, adder = self.server.accept()
            self.client = s_client
            self.adder = adder
            raw_data = self.client.recv(11)
            if raw_data[1] == 0:
                self.status = True
                print('机械腿成功连接')
                break
            else:
                s_client.close()

    def move(self, s_id, s_angle, s_time):
        servo_cmd = [b'\x55', b'\x55', struct.pack('B', len(s_id) * 3 + 5), struct.pack('B', self.move_cmd),
                     struct.pack('B', len(s_id)), struct.pack('H', s_time)]
        for i in range(len(s_id)):
            servo_cmd.append(struct.pack('B', s_id[i]))
            servo_cmd.append(struct.pack('H', int(s_angle[i])))
        servo_cmd = b''.join(servo_cmd)
        self.client.send(servo_cmd)

        for index, element in enumerate(s_id):
            self.position[element] = round(s_angle[index], 2)
        return

    def pos(self, s_id):
        for i in range(len(s_id)):
            servo_cmd = [b'\x55', b'\x55', struct.pack('B', 4), b'\x15', struct.pack('B', 1), struct.pack('B', s_id[i])]
            servo_cmd = b''.join(servo_cmd)
            self.client.send(servo_cmd)
            pos_data = self.client.send(8)
            try:
                self.back_data[s_id[i] - 1] = struct.unpack('H', pos_data[6:8])[0]
            except Exception as e:
                print(e)
        return

    def record(self):
        if self.status:
            self.pos([3, 6])
            time.sleep(0.1)
            with open(servo_address + "/servo3.txt", "a") as f_servo3:
                f_servo3.write(str(self.back_data[3 - 1]))
                f_servo3.write(" ")
            with open(servo_address + "/servo3_time.txt", "a") as f_servo3_time:
                f_servo3_time.write(str(time.time() - self.time_start))
                f_servo3_time.write(" ")

            with open(servo_address + "/servo6.txt", "a") as f_servo6:
                f_servo6.write(str(self.back_data[6 - 1]))
                f_servo6.write(" ")
            with open(servo_address + "/servo6_time.txt", "a") as f_servo6_time:
                f_servo6_time.write(str(time.time() - self.time_start))
                f_servo6_time.write(" ")
        return

    def reset(self):
        self.move([1, 2, 3, 4, 5, 6], [500, 500, 500, 500, 500, 500], 500)



# coding utf-8
from Control import *
from Display import *
from threading import Thread

"""   清空临时数据文件夹  """
setdir("./Data/Data_tmp")

""" 开启树莓派反向代理"""
# os.system("setsid sunny clientid 151150344278 &")

if __name__ == '__main__':

    connect_thread = Thread(target=connect_all, args=())
    connect_thread.setDaemon(True)
    connect_thread.start()

    print("Simulation open?")
    print('[Y] yes [N] no')
    print('>>', end=' ')
    if input() == 'Y':
        simulation_thread = Thread(target=simulation_start, args=())
        simulation_thread.setDaemon(True)
        simulation_thread.start()

    while 1:
        if (1 in Gyro.client_index) and (3 in Gyro.client_index) and robot.status:
            gyro_1_thread = Thread(target=gyro_1.activate, args=())
            gyro_1_thread.setDaemon(True)
            gyro_1_thread.start()

            gyro_3_thread = Thread(target=gyro_3.activate, args=())
            gyro_3_thread.setDaemon(True)
            gyro_3_thread.start()
            break

    control_thread = Thread(target=control_main, args=())
    control_thread.setDaemon(True)
    control_thread.start()

    display_main()



# coding:utf-8
from threading import Thread
import socket
from Gyro import *

'''Input: 连续字符串  Output： 二进制数据数组     '''

data_base_zzm___ = './Data/Data_base/zzm/大腿数据_8/'


def format_data(gyro_data):
    gyro_data = gyro_data.split()
    data = []
    for i in range(len(gyro_data)):
        data_temp = int(float(gyro_data[i]) / 180 * 32768)
        data.append(struct.pack('h', data_temp))
    return data


def pack_data(data_roll, data_yaw, data_pitch):
    data = []
    len_min = min(len(data_roll), len(data_yaw), len(data_pitch))
    for i in range(len_min):
        data.append(b''.join([b'\x55\x53', data_roll[i], data_pitch[i], data_yaw[i], b'\x00\x00\x00']))
    return data


def simulated_gyro(gyro_data, s_id):
    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  # 创建客户端套接字
    client.connect(Adder)  # 发起TCP连接

    client.send(s_id)
    time.sleep(0.1)

    for i in gyro_data:
        client.send(i)
        time.sleep(0.1)


def simulated_robot(s_id):
    client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  # 创建客户端套接字
    client.connect(Adder)  # 发起TCP连接

    client.send(s_id)
    time.sleep(0.1)
    while 1:
        pass


def simulation_start():
    simulated_gyro_1 = Thread(target=simulated_gyro, args=(gyro_data_1, Id_1))
    simulated_gyro_1.setDaemon(True)
    simulated_gyro_1.start()

    simulated_gyro_4 = Thread(target=simulated_gyro, args=(gyro_data_4, Id_3))
    simulated_gyro_4.setDaemon(True)
    simulated_gyro_4.start()

    # simulated_Robot = Thread(target=simulated_robot, args=(Id_robot,))
    # simulated_Robot.setDaemon(True)
    # simulated_Robot.start()


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    while 1:
        try:
            s.connect(('198.168.1.1', 80))
            ip = s.getsockname()[0]
            break
        finally:
            s.close()
    return ip


HOST = get_ip()
PORT = 8000  # 通信端口号
BUF_siz = 1024  # 接收数据缓冲大小
Adder = (HOST, PORT)

Id_1 = b'\x55\x01\x01\x00\x00\x00\x00\x00\x00\x00'
Id_2 = b'\x55\x01\x02\x00\x00\x00\x00\x00\x00\x00'
Id_3 = b'\x55\x01\x03\x00\x00\x00\x00\x00\x00\x00'
Id_4 = b'\x55\x01\x04\x00\x00\x00\x00\x00\x00\x00'

Id_robot = b'\x55\x00\x01\x00\x00\x00\x00\x00\x00\x00'

with open(data_base_zzm___ + 'example_roll1.txt', 'r') as f:
    gyro_data_roll1 = f.read()
    gyro_data_roll1 = format_data(gyro_data_roll1)
with open(data_base_zzm___ + 'gyro_yaw1.txt', 'r') as f:
    gyro_data_yaw1 = f.read()
    gyro_data_yaw1 = format_data(gyro_data_yaw1)
with open(data_base_zzm___ + 'gyro_pitch1.txt', 'r') as f:
    gyro_data_pitch1 = f.read()
    gyro_data_pitch1 = format_data(gyro_data_pitch1)

gyro_data_1 = pack_data(gyro_data_roll1, gyro_data_yaw1, gyro_data_pitch1)

with open(data_base_zzm___ + 'example_roll4.txt', 'r') as f:
    gyro_data_roll4 = f.read()
    gyro_data_roll4 = format_data(gyro_data_roll4)
with open(data_base_zzm___ + 'gyro_yaw4.txt', 'r') as f:
    gyro_data_yaw4 = f.read()
    gyro_data_yaw4 = format_data(gyro_data_yaw4)
with open(data_base_zzm___ + 'gyro_pitch4.txt', 'r') as f:
    gyro_data_pitch4 = f.read()
    gyro_data_pitch4 = format_data(gyro_data_pitch4)

gyro_data_4 = pack_data(gyro_data_roll4, gyro_data_yaw4, gyro_data_pitch4)

if __name__ == '__main__':

    print('IP: ', HOST)
    print('Simulation start')

    simulation_start()

    while True:
        pass



import struct
import time


class Gyro:
    """所有陀螺仪的基类"""

    """陀螺仪参数设置"""
    setup_open = b'\xFF\xAA\x69\x88\xB5'
    setup_zero = b'\xFF\xAA\x01\x08\x00'
    setup_close = b'\xFF\xAA\x00\x00\x00'
    setup_speed_100Hz = b'\xFF\xAA\x03\x09\x00'
    setup_data_angel = b'\xFF\xAA\x02\x08\x00'  # 设置只要角度
    setup_data_all = b'\xFF\xAA\x02\x0A\x00'  # 设置数据全部输出

    """数据存放路径"""
    gyro_address = r".\Data\Data_tmp"
    """管理客户端在线情况"""
    client_index = []

    def __init__(self, server):
        self.server = server

        self.name = None
        self.client = None
        self.adder = None

        """陀螺仪数据"""
        self.temper = 0
        self.roll = 0
        self.pitch = 0
        self.yaw = 0
        self.ax = 0
        self.ay = 0
        self.az = 0
        self.fps = 0
        self.time_angel = 0
        self.time_acc = 0
        self.time_start = 0
        self.roll_before = 0

    def connect(self, s_id):
        while 1:
            s_client, adder = self.server.accept()
            self.client = s_client
            self.adder = adder
            try:
                raw_data = self.client.recv(11)
                if s_id == raw_data[2] and raw_data[1] == 1:
                    self.name = raw_data[2]
                    self.time_start = time.time()
                    print("陀螺仪ID:", self.name, '成功连接')
                    Gyro.client_index.append(s_id)
                    break
                else:
                    s_client.close()
                    break
            except Exception as e:
                print(e)

    def calibration(self, speed, data):
        self.client.send(Gyro.setup_open)
        time.sleep(0.1)
        self.client.send(Gyro.setup_zero)
        time.sleep(0.1)
        self.client.send(speed)
        time.sleep(0.1)
        self.client.send(data)
        time.sleep(0.1)
        self.client.send(Gyro.setup_close)
        time.sleep(0.1)

    def refresh(self, raw_data):
        try:
            self.roll_before = self.roll
            if raw_data[1] == 83:
                self.roll = struct.unpack('h', raw_data[2:4])[0] / 32768 * 180
                self.pitch = struct.unpack('h', raw_data[4:6])[0] / 32768 * 180
                self.yaw = struct.unpack('h', raw_data[6:8])[0] / 32768 * 180
                self.time_angel = time.time()
            if raw_data[1] == 81:
                self.ax = struct.unpack('h', raw_data[2:4])[0] / 32768 * 16 * 9.8
                self.ay = struct.unpack('h', raw_data[4:6])[0] / 32768 * 16 * 9.8
                self.az = struct.unpack('h', raw_data[6:8])[0] / 32768 * 16 * 9.8
                self.time_acc = time.time()
        except Exception as e:
            print(e)

    def record(self, raw_data):
        if raw_data[1] == 83:
            with open(Gyro.gyro_address + r"\gyro_roll" + str(self.name) + ".txt", "a") as f_gyro_roll:
                f_gyro_roll.write(str(self.roll))
                f_gyro_roll.write(" ")
            with open(Gyro.gyro_address + r"\gyro_pitch" + str(self.name) + ".txt", "a") as f_gyro_pitch:
                f_gyro_pitch.write(str(self.pitch))
                f_gyro_pitch.write(" ")
            with open(Gyro.gyro_address + r"\gyro_yaw" + str(self.name) + ".txt", "a") as f_gyro_yaw:
                f_gyro_yaw.write(str(self.yaw))
                f_gyro_yaw.write(" ")
            with open(Gyro.gyro_address + r"\gyro_angel_time" + str(self.name) + ".txt",
                      "a") as f_angel_time:
                f_angel_time.write(str(self.time_angel - self.time_start))
                f_angel_time.write(" ")

        if raw_data[1] == 81:
            with open(Gyro.gyro_address + r"\gyro_ax" + str(self.name) + ".txt", "a") as f_gyro_ax:
                f_gyro_ax.write(str(self.ax))
                f_gyro_ax.write(" ")
            with open(Gyro.gyro_address + r"\gyro_ay" + str(self.name) + ".txt", "a") as f_gyro_yx:
                f_gyro_yx.write(str(self.ay))
                f_gyro_yx.write(" ")
            with open(Gyro.gyro_address + r"\gyro_az" + str(self.name) + ".txt", "a") as f_gyro_zx:
                f_gyro_zx.write(str(self.az))
                f_gyro_zx.write(" ")
            with open(Gyro.gyro_address + r"\gyro_a_time" + str(self.name) + ".txt", "a") as f_a_time:
                f_a_time.write(str(self.time_acc - self.time_start))
                f_a_time.write(" ")

    def activate(self):
        self.calibration(self.setup_speed_100Hz, self.setup_data_all)
        self.time_start = time.time()
        counter = self.time_start
        while True:
            raw_data = self.client.recv(11)
            self.refresh(raw_data)
            self.record(raw_data)
            if (time.time() - counter) > 1:
                self.fps = 0
                counter = time.time()
            else:
                self.fps += 1



import curses
from Simulation import *
from Control import *


def display_main():
    stdscr = curses.initscr()
    curses.noecho()
    stdscr.nodelay(True)
    stdscr.addstr(1, 25, '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>system runing<<<<<<<<<<<<<<<<<<<<<<<<<<', curses.A_REVERSE)
    stdscr.refresh()
    counter = 0
    while 1:
        s_line = 13
        stdscr.addstr(2, 25, '          Sampling {0}'.format(counter))
        stdscr.addstr(4, 25, '                Servo 3 {0}                  Servo 6 {1}            '.format(robot.position[3], robot.position[6]))
        stdscr.addstr(5, 25, '          Back 1000 ---> Front 0         Back 0 ---> Front 1000     ')
        stdscr.addstr(7, 25, '                Servo 2 {0}                  Servo 5 {1}            '.format(robot.position[2], robot.position[5]))
        stdscr.addstr(8, 25, '          Back 0 ---> Front 1000         Back 1000 ---> Front 0     ')
        stdscr.addstr(10, 25,'                Servo 1 {0}                  Servo 4 {1}            '.format(robot.position[1], robot.position[4]))
        stdscr.addstr(11, 25,'          Left 0 ---> Right 1000         Left 0 ---> Right 1000     ')
        for i in Gyro.client_index:
            exec("stdscr.addstr(s_line, 25, 'gyro {0} roll:'+str(round(gyro_{0}.roll, 2)) + '   ', curses.A_REVERSE)".format(i))
            exec("stdscr.addstr(s_line, 50, 'gyro {0} row:'+str(round(gyro_{0}.yaw, 2)) + '   ', curses.A_REVERSE)".format(i))
            exec("stdscr.addstr(s_line, 75, 'gyro {0} pitch:'+str(round(gyro_{0}.pitch, 2)) + '   ', curses.A_REVERSE)".format(i))
            s_line = s_line + 2

        c = stdscr.getch()

        if c == ord('b'):
            break

        if c == ord('s'):
            break

        counter = counter + 1

    curses.nocbreak()
    stdscr.keypad(False)
    curses.echo()
    curses.endwin()


if __name__ == "__main__":

    display_main()



from setuptools import setup, find_packages

setup(
    name="leg2",
    version="1.0",
    author="mark-cyber-cmd",
    author_email="dingyiming19991203@163.com",
    description="Learn to Pack Python Module  -->公众号：等离子工作室",

    # 项目主页
    url="https://www.baidu.com",

    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=find_packages()
)



import numpy as np
import os
import shutil
import keyboard
import socket
import pywifi
from Robot import *
from Gyro import *

# 保存包中写义的常量
from pywifi import const

"""                  神经网络控制参数                 """
w1 = np.loadtxt("./Data/BP_net/Net6/w1.txt", delimiter=" ", dtype="float")
w2 = np.loadtxt("./Data/BP_net/Net6/w2.txt", delimiter=" ", dtype="float")
b1 = np.loadtxt("./Data/BP_net/Net6/b1.txt", delimiter=" ", dtype="float")
b2 = np.loadtxt("./Data/BP_net/Net6/b2.txt", delimiter=" ", dtype="float")
b1 = np.array([b1.tolist()]).T

"""                      控制参数                     """
con_time = 100  # ms
N = 1  # 控制采样频率相对指令的倍数
gravity = 0


def connect_all():
    global gyro_1, gyro_3, server
    while 1:
        s_client, adder = server.accept()
        try:
            raw_data = s_client.recv(11)
            if raw_data[1] == 1:
                s_id = raw_data[2]
                exec('gyro_{0}.client = s_client'.format(s_id))
                exec('gyro_{0}.adder = adder'.format(s_id))
                exec('gyro_{0}.name = s_id'.format(s_id))
                exec('Gyro.client_index.append({0})'.format(s_id))
                print('陀螺仪ID:', s_id, '成功连接')

            if raw_data[1] == 0:
                robot.client = s_client
                robot.adder = adder
                robot.status = True
                print('机械腿成功连接')
        except Exception as e:
            print(e)


def control_main():
    global gravity, gyro_1, gyro_3, robot

    # 默认舵机位置
    bus_data_s = [500, 500, 500, 500]

    time.sleep(1)
    while 1:
        # 导入陀螺仪数据，组成神经网络输入端数据
        # data_set = [[gyro_1.ax, gyro_1.ay, gyro_1.az, gyro_4.ax,
        #              gyro_4.ay, gyro_4.az, gyro_1.roll, gyro_1.pitch,
        #              gyro_1.yaw, gyro_4.roll, gyro_4.pitch, gyro_4.yaw]]
        # data_set = np.array(data_set).T

        # TODO 待测试神经网络代码，现用经验法判断
        # if settle_down(data_set):
        #     robot.servo_move([1, 4], [500, 500], 200)
        #     time.sleep(0.2)
        #     gravity = 0

        # 若一只腿抬起则将重心偏移致并双腿恢复到并排状态
        if gyro_3.roll - gyro_3.roll_before > 0.1 and gravity == 0:
            robot.move([1, 4], [440, 440], 200)
            time.sleep(0.2)
            robot.move([2, 3, 5, 6], [500, 500, 500, 500], 500)  # 步距补偿
            time.sleep(0.5)
            gravity = 1

        # 在gravity=1状态下跟随动作并检测是否落脚
        if abs(gyro_3.roll + gyro_1.roll) > 1 and gravity == 1 and abs(gyro_3.roll) <= 135:
            bus_data_s[0] = \
                500 + gyro_3.roll / 270 * 1000
            bus_data_s[1] = \
                500 + gyro_3.roll / 270 * 1000
            bus_data_s[2] = \
                500 + gyro_3.roll / 270 * 1000
            bus_data_s[3] = \
                500 + gyro_3.roll / 270 * 1000
            robot.move([2, 3, 5, 6], bus_data_s, con_time)
            time.sleep(con_time / 1000)

            # 此处使用8倍关系经验判断落脚阈值为5度
            if gyro_3.roll + 8 * gyro_1.roll < 5:
                robot.move([1, 4], [500, 500], 200)
                time.sleep(0.2)
                gravity = 0

        # 相同的操作在另外一只脚上的镜像
        if gyro_1.roll - gyro_1.roll_before > 0.1 and gravity == 0:
            robot.move([1, 4], [560, 560], 200)
            time.sleep(0.2)
            robot.move([2, 3, 5, 6], [500, 500, 500, 500], 500)
            time.sleep(0.5)
            gravity = -1

        # 相同的操作在另外一只脚上的跟随
        if abs(gyro_3.roll + gyro_1.roll) > 1 and gravity == -1 and abs(gyro_1.roll) < 135:
            bus_data_s[0] = \
                500 - gyro_1.roll / 270 * 1000
            bus_data_s[1] = \
                500 - gyro_1.roll / 270 * 1000
            bus_data_s[2] = \
                500 - gyro_1.roll / 270 * 1000
            bus_data_s[3] = \
                500 - gyro_1.roll / 270 * 1000
            robot.move([2, 3, 5, 6], bus_data_s, con_time)
            time.sleep(con_time / 1000)

            # 相同的操作以8倍关系经验判断落脚阈值为5度
            if gyro_1.roll + 8 * gyro_3.roll < 5:
                robot.move([1, 4], [500, 500], 200)
                time.sleep(0.2)
                gravity = 0


def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    while 1:
        try:
            s.connect(('198.168.1.1', 80))
            ip = s.getsockname()[0]
            break
        finally:
            s.close()
    print("服务器IP为：", ip)
    return ip


# 创建激活函数sigmoid
def sigmoid(z):
    return 2 / (1 + np.exp(-2 * z)) - 1


def settle_down(data):
    step1 = np.dot(w1, data)
    step2 = sigmoid(step1 - b1)
    step3 = np.dot(w2, step2)
    step4 = sigmoid(step3 - b2)
    output = np.heaviside(step4 - 0.5, 1)
    return output


def setdir(filepath):
    if not os.path.exists(filepath):
        os.mkdir(filepath)
    else:
        shutil.rmtree(filepath)
        os.mkdir(filepath)


def key_scan(flag):
    try:
        if keyboard.is_pressed('space'):
            print('记录落脚一次')
            flag = 1
    except Exception as e:
        print(e)
    return flag


def connect_wifi(ssid, key):
    wifi = pywifi.PyWiFi()  # 创建一个wifi对象
    ifaces = wifi.interfaces()[0]  # 取第一个无限网卡
    # print(ifaces.name())  # 输出无线网卡名称
    ifaces.disconnect()  # 断开网卡连接
    time.sleep(3)  # 缓冲3秒

    profile = pywifi.Profile()  # 配置文件
    profile.ssid = ssid  # wifi名称
    profile.auth = const.AUTH_ALG_OPEN  # 需要密码
    profile.akm.append(const.AKM_TYPE_WPA2PSK)  # 加密类型
    profile.cipher = const.CIPHER_TYPE_CCMP  # 加密单元
    profile.key = key  # wifi密码

    ifaces.remove_all_network_profiles()  # 删除其他配置文件
    tmp_profile = ifaces.add_network_profile(profile)  # 加载配置文件

    ifaces.connect(tmp_profile)  # 连接
    time.sleep(5)  # 尝试10秒能否成功连接
    if ifaces.status() == const.IFACE_CONNECTED:
        print("连接WiFi:", ssid, "成功")
        return True
    else:
        print("连接WiFi:", ssid, "失败")
        return False


def server_init(port):
    server_s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    server_s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)  # 不经过WAIT_TIME，直接关闭
    # server_s.setblocking(False)  # 设置非阻塞编程
    server_s.bind((get_ip(), port))  # 自动获取本机ip 并在8000端口创建服务器
    server_s.listen(5)
    print("服务器准备就绪,等待客户端上线..........\r\n")
    return server_s


"""           建立一个服务端            """
server = server_init(8000)
"""            生成控制对象               """
gyro_1 = Gyro(server)
gyro_3 = Gyro(server)
robot = Robot(server)
gravity = 0
